@@ -50,7 +50,7 @@ from swift.common.swob import Request, Response, HeaderKeyDict, Range, \
     HTTPException, HTTPRequestedRangeNotSatisfiable
 from swift.common.request_helpers import strip_sys_meta_prefix, \
     strip_user_meta_prefix, is_user_meta, is_sys_meta, is_sys_or_user_meta
-from swift.common.storage_policy import POLICY_INDEX, POLICY, POLICIES
+from swift.common.storage_policy import POLICIES
 
 
 def update_headers(response, headers):
@@ -162,7 +162,8 @@ def headers_to_container_info(headers, status_int=HTTP_OK):
         'object_count': headers.get('x-container-object-count'),
         'bytes': headers.get('x-container-bytes-used'),
         'versions': headers.get('x-versions-location'),
-        'storage_policy': headers.get(POLICY_INDEX.lower(), '0'),
+        'storage_policy': headers.get('X-Backend-Storage-Policy-Index'.lower(),
+                                      '0'),
         'cors': {
             'allow_origin': meta.get('access-control-allow-origin'),
             'expose_headers': meta.get('access-control-expose-headers'),
@@ -1208,14 +1209,18 @@ class Controller(object):
             pass
         # if a backend policy index is present in resp headers, translate it
         # here with the friendly policy name
-        if POLICY_INDEX in res.headers and is_success(res.status_int):
-            policy = POLICIES.get_by_index(res.headers[POLICY_INDEX])
+        if 'X-Backend-Storage-Policy-Index' in res.headers and \
+                is_success(res.status_int):
+            policy = \
+                POLICIES.get_by_index(
+                    res.headers['X-Backend-Storage-Policy-Index'])
             if policy:
-                res.headers[POLICY] = policy.name
+                res.headers['X-Storage-Policy'] = policy.name
             else:
                 self.app.logger.error(
                     'Could not translate %s (%r) from %r to policy',
-                    POLICY_INDEX, res.headers[POLICY_INDEX], path)
+                    'X-Backend-Storage-Policy-Index',
+                    res.headers['X-Backend-Storage-Policy-Index'], path)
         return res
 
     def is_origin_allowed(self, cors_info, origin):
