@@ -38,7 +38,7 @@ from eventlet.timeout import Timeout
 from swift.common.utils import (
     clean_content_type, config_true_value, ContextPool, csv_append,
     GreenAsyncPile, GreenthreadSafeIterator, json, Timestamp,
-    normalize_delete_at_timestamp, public, quorum_size)
+    normalize_delete_at_timestamp, public, quorum_size, get_expirer_container)
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_metadata, check_object_creation, \
     check_copy_from_header
@@ -56,8 +56,8 @@ from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPNotFound, \
     HTTPPreconditionFailed, HTTPRequestEntityTooLarge, HTTPRequestTimeout, \
     HTTPServerError, HTTPServiceUnavailable, Request, \
     HTTPClientDisconnect, HTTPNotImplemented
-from swift.common.storage_policy import POLICY_INDEX
-from swift.common.request_helpers import is_user_meta
+from swift.common.request_helpers import is_sys_or_user_meta, is_sys_meta, \
+    remove_items, copy_header_subset
 
 
 def copy_headers_into(from_r, to_r):
@@ -68,7 +68,7 @@ def copy_headers_into(from_r, to_r):
     """
     pass_headers = ['x-delete-at']
     for k, v in from_r.headers.items():
-        if is_user_meta('object', k) or k.lower() in pass_headers:
+        if is_sys_or_user_meta('object', k) or k.lower() in pass_headers:
             to_r.headers[k] = v
 
 
@@ -168,13 +168,13 @@ class ObjectController(Controller):
         :param partition: ring partition to yield nodes for
         """
 
-        primary_nodes = ring.get_part_nodes(partition)
-        num_locals = self.app.write_affinity_node_count(len(primary_nodes))
         is_local = self.app.write_affinity_is_local_fn
-
         if is_local is None:
             return self.app.iter_nodes(ring, partition)
 
+        primary_nodes = ring.get_part_nodes(partition)
+        num_locals = self.app.write_affinity_node_count(len(primary_nodes))
+
         all_nodes = itertools.chain(primary_nodes,
                                     ring.get_more_nodes(partition))
         first_n_local_nodes = list(itertools.islice(
@@ -197,10 +197,10 @@ class ObjectController(Controller):
             self.account_name, self.container_name, req)
         req.acl = container_info['read_acl']
         # pass the policy index to storage nodes via req header
-        policy_index = req.headers.get(POLICY_INDEX,
+        policy_index = req.headers.get('X-Backend-Storage-Policy-Index',
                                        container_info['storage_policy'])
         obj_ring = self.app.get_object_ring(policy_index)
-        req.headers[POLICY_INDEX] = policy_index
+        req.headers['X-Backend-Storage-Policy-Index'] = policy_index
         if 'swift.authorize' in req.environ:
             aresp = req.environ['swift.authorize'](req)
             if aresp:
@@ -235,15 +235,6 @@ class ObjectController(Controller):
     @delay_denial
     def POST(self, req):
         """HTTP POST request handler."""
-        if 'x-delete-after' in req.headers:
-            try:
-                x_delete_after = int(req.headers['x-delete-after'])
-            except ValueError:
-                return HTTPBadRequest(request=req,
-                                      content_type='text/plain',
-                                      body='Non-integer X-Delete-After')
-            req.headers['x-delete-at'] = normalize_delete_at_timestamp(
-                time.time() + x_delete_after)
         if self.app.object_post_as_copy:
             req.method = 'PUT'
             req.path_info = '/v1/%s/%s/%s' % (
@@ -279,36 +270,22 @@ class ObjectController(Controller):
                     return aresp
             if not containers:
                 return HTTPNotFound(request=req)
-            if 'x-delete-at' in req.headers:
-                try:
-                    x_delete_at = normalize_delete_at_timestamp(
-                        int(req.headers['x-delete-at']))
-                    if int(x_delete_at) < time.time():
-                        return HTTPBadRequest(
-                            body='X-Delete-At in past', request=req,
-                            content_type='text/plain')
-                except ValueError:
-                    return HTTPBadRequest(request=req,
-                                          content_type='text/plain',
-                                          body='Non-integer X-Delete-At')
-                req.environ.setdefault('swift.log_info', []).append(
-                    'x-delete-at:%s' % x_delete_at)
-                delete_at_container = normalize_delete_at_timestamp(
-                    int(x_delete_at) /
-                    self.app.expiring_objects_container_divisor *
-                    self.app.expiring_objects_container_divisor)
-                delete_at_part, delete_at_nodes = \
-                    self.app.container_ring.get_nodes(
-                        self.app.expiring_objects_account, delete_at_container)
-            else:
-                delete_at_container = delete_at_part = delete_at_nodes = None
+
+            try:
+                req, delete_at_container, delete_at_part, \
+                    delete_at_nodes = self._config_obj_expiration(req)
+            except ValueError as e:
+                return HTTPBadRequest(request=req, content_type='text/plain',
+                                      body=str(e))
+
             # pass the policy index to storage nodes via req header
-            policy_index = req.headers.get(POLICY_INDEX,
+            policy_index = req.headers.get('X-Backend-Storage-Policy-Index',
                                            container_info['storage_policy'])
             obj_ring = self.app.get_object_ring(policy_index)
-            req.headers[POLICY_INDEX] = policy_index
+            req.headers['X-Backend-Storage-Policy-Index'] = policy_index
             partition, nodes = obj_ring.get_nodes(
                 self.account_name, self.container_name, self.object_name)
+
             req.headers['X-Timestamp'] = Timestamp(time.time()).internal
 
             headers = self._backend_requests(
@@ -447,6 +424,43 @@ class ObjectController(Controller):
             bodies.append('')
         return statuses, reasons, bodies, etags
 
+    def _config_obj_expiration(self, req):
+        delete_at_container = None
+        delete_at_part = None
+        delete_at_nodes = None
+
+        if 'x-delete-after' in req.headers:
+            try:
+                x_delete_after = int(req.headers['x-delete-after'])
+            except ValueError:
+                raise ValueError('Non-integer X-Delete-After')
+
+            req.headers['x-delete-at'] = normalize_delete_at_timestamp(
+                time.time() + x_delete_after)
+
+        if 'x-delete-at' in req.headers:
+            try:
+                x_delete_at = int(normalize_delete_at_timestamp(
+                    int(req.headers['x-delete-at'])))
+            except ValueError:
+                raise ValueError('Non-integer X-Delete-At')
+
+            if x_delete_at < time.time():
+                raise ValueError('X-Delete-At in past')
+
+            req.environ.setdefault('swift.log_info', []).append(
+                'x-delete-at:%s' % x_delete_at)
+
+            delete_at_container = get_expirer_container(
+                x_delete_at, self.app.expiring_objects_container_divisor,
+                self.account_name, self.container_name, self.object_name)
+
+            delete_at_part, delete_at_nodes = \
+                self.app.container_ring.get_nodes(
+                    self.app.expiring_objects_account, delete_at_container)
+
+        return req, delete_at_container, delete_at_part, delete_at_nodes
+
     @public
     @cors_validation
     @delay_denial
@@ -458,11 +472,12 @@ class ObjectController(Controller):
                                   body='If-None-Match only supports *')
         container_info = self.container_info(
             self.account_name, self.container_name, req)
-        policy_index = req.headers.get(POLICY_INDEX,
+        policy_index = req.headers.get('X-Backend-Storage-Policy-Index',
                                        container_info['storage_policy'])
         obj_ring = self.app.get_object_ring(policy_index)
+
         # pass the policy index to storage nodes via req header
-        req.headers[POLICY_INDEX] = policy_index
+        req.headers['X-Backend-Storage-Policy-Index'] = policy_index
         container_partition = container_info['partition']
         containers = container_info['nodes']
         req.acl = container_info['write_acl']
@@ -472,8 +487,10 @@ class ObjectController(Controller):
             aresp = req.environ['swift.authorize'](req)
             if aresp:
                 return aresp
+
         if not containers:
             return HTTPNotFound(request=req)
+
         try:
             ml = req.message_length()
         except ValueError as e:
@@ -484,17 +501,10 @@ class ObjectController(Controller):
                                       body=str(e))
         if ml is not None and ml > constraints.MAX_FILE_SIZE:
             return HTTPRequestEntityTooLarge(request=req)
-        if 'x-delete-after' in req.headers:
-            try:
-                x_delete_after = int(req.headers['x-delete-after'])
-            except ValueError:
-                return HTTPBadRequest(request=req,
-                                      content_type='text/plain',
-                                      body='Non-integer X-Delete-After')
-            req.headers['x-delete-at'] = normalize_delete_at_timestamp(
-                time.time() + x_delete_after)
+
         partition, nodes = obj_ring.get_nodes(
             self.account_name, self.container_name, self.object_name)
+
         # do a HEAD request for container sync and checking object versions
         if 'x-timestamp' in req.headers or \
                 (object_versions and not
@@ -507,6 +517,7 @@ class ObjectController(Controller):
             hresp = self.GETorHEAD_base(
                 hreq, _('Object'), obj_ring, partition,
                 hreq.swift_entity_path)
+
         # Used by container sync feature
         if 'x-timestamp' in req.headers:
             try:
@@ -522,6 +533,7 @@ class ObjectController(Controller):
             req.headers['X-Timestamp'] = req_timestamp.internal
         else:
             req.headers['X-Timestamp'] = Timestamp(time.time()).internal
+
         # Sometimes the 'content-type' header exists, but is set to None.
         content_type_manually_set = True
         detect_content_type = \
@@ -585,8 +597,9 @@ class ObjectController(Controller):
             source_header = '/%s/%s/%s/%s' % (ver, acct,
                                               src_container_name, src_obj_name)
             source_req = req.copy_get()
+
             # make sure the source request uses it's container_info
-            source_req.headers.pop(POLICY_INDEX, None)
+            source_req.headers.pop('X-Backend-Storage-Policy-Index', None)
             source_req.path_info = source_header
             source_req.headers['X-Newest'] = 'true'
             orig_obj_name = self.object_name
@@ -596,6 +609,7 @@ class ObjectController(Controller):
             sink_req = Request.blank(req.path_info,
                                      environ=req.environ, headers=req.headers)
             source_resp = self.GET(source_req)
+
             # This gives middlewares a way to change the source; for example,
             # this lets you COPY a SLO manifest and have the new object be the
             # concatenation of the segments (like what a GET request gives
@@ -620,15 +634,23 @@ class ObjectController(Controller):
             if sink_req.content_length > constraints.MAX_FILE_SIZE:
                 return HTTPRequestEntityTooLarge(request=req)
             sink_req.etag = source_resp.etag
+
             # we no longer need the X-Copy-From header
             del sink_req.headers['X-Copy-From']
             if not content_type_manually_set:
                 sink_req.headers['Content-Type'] = \
                     source_resp.headers['Content-Type']
-            if not config_true_value(
+            if config_true_value(
                     sink_req.headers.get('x-fresh-metadata', 'false')):
+                # post-as-copy: ignore new sysmeta, copy existing sysmeta
+                condition = lambda k: is_sys_meta('object', k)
+                remove_items(sink_req.headers, condition)
+                copy_header_subset(source_resp, sink_req, condition)
+            else:
+                # copy/update existing sysmeta and user meta
                 copy_headers_into(source_resp, sink_req)
                 copy_headers_into(req, sink_req)
+
             # copy over x-static-large-object for POSTs and manifest copies
             if 'X-Static-Large-Object' in source_resp.headers and \
                     req.params.get('multipart-manifest') == 'get':
@@ -637,28 +659,12 @@ class ObjectController(Controller):
 
             req = sink_req
 
-        if 'x-delete-at' in req.headers:
-            try:
-                x_delete_at = normalize_delete_at_timestamp(
-                    int(req.headers['x-delete-at']))
-                if int(x_delete_at) < time.time():
-                    return HTTPBadRequest(
-                        body='X-Delete-At in past', request=req,
-                        content_type='text/plain')
-            except ValueError:
-                return HTTPBadRequest(request=req, content_type='text/plain',
-                                      body='Non-integer X-Delete-At')
-            req.environ.setdefault('swift.log_info', []).append(
-                'x-delete-at:%s' % x_delete_at)
-            delete_at_container = normalize_delete_at_timestamp(
-                int(x_delete_at) /
-                self.app.expiring_objects_container_divisor *
-                self.app.expiring_objects_container_divisor)
-            delete_at_part, delete_at_nodes = \
-                self.app.container_ring.get_nodes(
-                    self.app.expiring_objects_account, delete_at_container)
-        else:
-            delete_at_container = delete_at_part = delete_at_nodes = None
+        try:
+            req, delete_at_container, delete_at_part, \
+                delete_at_nodes = self._config_obj_expiration(req)
+        except ValueError as e:
+            return HTTPBadRequest(request=req, content_type='text/plain',
+                                  body=str(e))
 
         node_iter = GreenthreadSafeIterator(
             self.iter_nodes_local_first(obj_ring, partition))
@@ -777,11 +783,11 @@ class ObjectController(Controller):
         container_info = self.container_info(
             self.account_name, self.container_name, req)
         # pass the policy index to storage nodes via req header
-        policy_index = req.headers.get(POLICY_INDEX,
+        policy_index = req.headers.get('X-Backend-Storage-Policy-Index',
                                        container_info['storage_policy'])
         obj_ring = self.app.get_object_ring(policy_index)
         # pass the policy index to storage nodes via req header
-        req.headers[POLICY_INDEX] = policy_index
+        req.headers['X-Backend-Storage-Policy-Index'] = policy_index
         container_partition = container_info['partition']
         containers = container_info['nodes']
         req.acl = container_info['write_acl']
@@ -838,7 +844,8 @@ class ObjectController(Controller):
                 policy_idx = container_info['storage_policy']
                 obj_ring = self.app.get_object_ring(policy_idx)
                 # pass the policy index to storage nodes via req header
-                new_del_req.headers[POLICY_INDEX] = policy_idx
+                new_del_req.headers['X-Backend-Storage-Policy-Index'] = \
+                    policy_idx
                 container_partition = container_info['partition']
                 containers = container_info['nodes']
                 new_del_req.acl = container_info['write_acl']
